#! /usr/bin/env python


from ..common.device_lock import Device_Lock
from .device_zwave_base import Device_ZWave_Base

state_map = {0: "Unlocked", 100: "Locked", 101: "Unknown", 102: "Jammed"}


class Device_ZWave_Door_Lock(Device_Lock, Device_ZWave_Base):
    def __init__(self, container, device_info):
        Device_Lock.__init__(self, container, device_info.name, device_info.address)
        Device_ZWave_Base.__init__(self, device_info)

        value = device_info.get_property("ST", "value")
        if value:
            try:
                self.set_property("state", state_map.get(int(value), "Unknown"))
            except:
                pass

    def process_websocket_event(self, event):
        action = int(event.action)

        if event.control == "ST":
            self.set_property("state", state_map.get(action, "Unknown"))
        elif event.control == "ALARM":
            self.set_property("alarmcode", action)
            alarm_map = {1: "Master Code Changed", 2: "Tamper Code Entry Limit", 3: "Escutcheon Removed", 4: "Key/Manually Locked", 5: "Locked by Touch", 6: "Key/Manually Unlocked", 7: "Remote Locking Jammed Bolt", 8: "Remotely Locked", 9: "Remotely Unlocked", 10: "Deadbolt Jammed", 11: "Battery Too Low to Operate", 12: "Critical Low Battery", 13: "Low Battery", 14: "Automatically Locked", 15: "Automatic Locking Jammed Bolt", 16: "Remotely Power Cycled", 17: "Lock Handling Completed", 19: "User Deleted", 20: "User Added", 21: "Duplicate PIN", 22: "Jammed Bolt by Locking with Keypad", 23: "Locked by Keypad", 24: "Unlocked by Keypad", 25: "Keypad Attempt outside Schedule", 26: "Hardware Failure", 27: "Factory Reset"}
            self.set_property("alarm", alarm_map.get(action, "Unknown"))
        elif event.control == "BATLVL":
            self.set_property("batterylevel", action)
        elif event.control == "ERR":
            self.set_property("error", action)
        elif event.control == "USRNUM":
            self.set_property("usernumber", action)

    def lock(self):
        path = "nodes/" + self.address + "/cmd/SECMD/1"
        return self.send_request(path)

    def unlock(self):
        path = "nodes/" + self.address + "/cmd/SECMD/0"
        return self.send_request(path)



#! /usr/bin/env python


from ..common.device_notification import Device_Notification
from .device_zwave_base import Device_ZWave_Base


class Device_ZWave_Notification(Device_Notification, Device_ZWave_Base):
    def __init__(self, container, device_info):
        Device_Notification.__init__(self, container, device_info.name, device_info.address)
        Device_ZWave_Base.__init__(self, device_info)

        value = device_info.get_property("ST", "value")
        if value:
            try:
                self.set_property("notification", value)
            except:
                pass

    def process_websocket_event(self, event):
        if event.control == "ST":
            self.set_property("notification", event.action)
        elif event.control == "BATLVL":
            self.set_property("batterylevel", int(event.action))



#! /usr/bin/env python

import traceback
from datetime import datetime
import xml.etree.ElementTree as ET

""" 

Base Device

Common to all elements returned from rest/nodes/devices

device_type = switch,dimmer,contact,...

having a common device types eliminates the need to know the underlying technology being used

"""


from ...item_base import Item_Base


class Device_Base(Item_Base):
    def __init__(self, container, device_type, name, address):
        Item_Base.__init__(self, container, name)

        self.device_type = device_type
        self.address = address

        self.last_command_path = None # last command we sent to the device
        self.last_command_time = None # time last command sent
        self.last_command_retry_interval = 5 # if device status changes time to allow for retry

        self.add_property("communication_error_count",0) # number of time the device enters an alert state

    def __str__(self):
        return "Device {}, type {}, ID {}".format(
            self.name, self.device_type, self.address
        )

    def get_identifier(self):
        return self.address
    
    def send_request(self, path, timeout=None):
        self.last_command_path = path
        self.last_command_time = datetime.now()
        success,response = Item_Base.send_request(self,path,timeout)
        return success,response

    def process_websocket_event(self, event):
        if event.control == "_3":
            if event.action == "NE": # comms error
                self.set_property ('status','alert')
                self.communication_failure()
                #self.get_status()
            elif event.action == "CE": # comms error clear
                self.set_property ('status','ready')
                print('comms error cleareed!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!')

    def get_status(self): #isy status for node
        path = "status/" + self.address 
        success, response = self.send_request(path)

        if success:  # and response.status == 200:
            try:
                root = ET.fromstring(response)
                node = (root.find(".//property[@id='ERR']"))

                if int(node.attrib ["value"]) == 0:
                    self.set_property ('status','ready')
                elif int(node.attrib ["value"]) > 0:
                    self.set_property ('status','alert',True)  #always publish
                    self.communications_failed()

            except Exception:
                traceback.print_exc()

        return success, response

    def communication_failure(self): # called when we receive a websocket event with a NE comm error
        print("******************Communication Failure for {}",self.address)
        self.set_property("communication_error_count",self.get_property("communication_error_count")+1)

        if self.last_command_path is not None:
            if (datetime.now()-self.last_command_time).seconds <= self.last_command_retry_interval:
                self.send_request(self.last_command_path)


    def communications_failed(self): # called when device status returns device in comm failed state
        #print("!!!!!!!!!!!!!!!!!!!!!!Communications Failed for {}",self.address)
        pass




#! /usr/bin/env python


from .device_base import Device_Base

 
class Device_Notification(Device_Base):
    def __init__(self, container, name, address):
        Device_Base.__init__(self, container, "notification", name, address)

        self.add_property("notification", None)
        self.add_property("batterylevel", 0)




#! /usr/bin/env python


from ..common.device_barrier import Device_Barrier
from .device_zwave_base import Device_ZWave_Base

state_map = {0: "closed", 100: "open", 101: "unknown", 102: "stopped", 103: "closing", 104: "opening"}


class Device_ZWave_Barrier(Device_Barrier, Device_ZWave_Base):
    def __init__(self, container, device_info):
        Device_Barrier.__init__(self, container, device_info.name, device_info.address)
        Device_ZWave_Base.__init__(self, device_info)

        value = device_info.get_property("ST", "value")
        if value:
            try:
                self.set_property("barrier", int(value))
                if (int(value) > 0 and int(value) < 100):
                    self.set_property("state", "percent closed")
                else:
                    self.set_property("state", state_map.get(int(value), "unknown"))
            except:
                pass

    def process_websocket_event(self, event):
        action = int(event.action)

        if event.control == "ST":
            self.set_property("barrier", action)
            if (action > 0 and action < 100):
                self.set_property("state", "percent closed")
            else:
                self.set_property("state", state_map.get(action, "unknown"))



#! /usr/bin/env python


# process a node and assemble the info to build a program


class Program_Info(object):
    def __init__(self, node):

        self.node = node

        self.valid = False

        try:
            self.id = node.attrib["id"]
            self.name = node.find("name").text
            self.folder = node.attrib["folder"] == "true"  # program folder, not interested

            if self.folder is False:
                self.enabled = node.attrib["enabled"] == "true"
                self.last_run_time = node.find("lastRunTime").text
                self.last_finish_time = node.find("lastFinishTime").text

            self.valid = True

        except Exception as err:
            self.err = err

    def __repr__(self):
        if self.valid is True:
            return "Program: Name {} ID {}".format(self.name, self.id)
        else:
            return "Invalid program node: {}".format(self.err)




#! /usr/bin/env python

""" 

returns a device instance using node data from an zwave device, None if unable to create device

"""

from .device_zwave_controller import Device_ZWave_Controller
from .device_zwave_door_lock import Device_ZWave_Door_Lock
from .device_zwave_on_off_power_switch import Device_ZWave_On_Off_Power_Switch
from .device_zwave_energy_meter import Device_ZWave_Energy_Meter
from .device_zwave_barrier import Device_ZWave_Barrier
from .device_zwave_binary import Device_ZWave_Binary
from .device_zwave_notification import Device_ZWave_Notification


zwave_device_classes = {
    # "0": Device_ZWave_Uninitialized,
    # "101": Device_ZWave_Unknown,
    # "102": Device_ZWave_Alarm,
    # "103": Device_ZWave_AV_Control_Point,
    "104": Device_ZWave_Binary,
    # "105": Device_ZWave_Class_A_Motor_Control,
    # "106": Device_ZWave_Class_B_Motor_Control,
    # "107": Device_ZWave_Class_C_Motor_Control,
    # "108": Device_ZWave_Controller,
    # "109": Device_ZWave_Dimmer_Switch,
    # "110": Device_ZWave_Display,
    "111": Device_ZWave_Door_Lock,
    # "112": Device_ZWave_Doorbell,
    # "113": Device_ZWave_Entry_Control,
    # "114": Device_ZWave_Gateway,
    # "115": Device_ZWave_Installer_Tool,
    # "116": Device_ZWave_Motor_Multiposition,
    # "117": Device_ZWave_Climate_Sensor,
    # "118": Device_ZWave_Multilevel_Sensor,
    # "119": Device_ZWave_Multilevel_Switch,
    # "120": Device_ZWave_On_Off_Power_Strip,
    "121": Device_ZWave_On_Off_Power_Switch,
    # "122": Device_ZWave_On_Off_Scene_Switch,
    # "123": Device_ZWave_Open_Close_Valve,
    # "124": Device_ZWave_PC_Controller,
    # "125": Device_ZWave_Remote,
    # "126": Device_ZWave_Remote_Control,
    # "127": Device_ZWave_AV_Remote_Control,
    # "128": Device_ZWave_Simple_Remote_Control,
    # "129": Device_ZWave_Repeater,
    # "130": Device_ZWave_Residential_HRV,
    # "131": Device_ZWave_Satellite_Receiver,
    # "132": Device_ZWave_Satellite_Receiver,
    # "133": Device_ZWave_Scene_Controller,
    # "134": Device_ZWave_Scene_Switch,
    # "135": Device_ZWave_Security_Panel,
    # "136": Device_ZWave_Set-Top_Box,
    # "137": Device_ZWave_Siren,
    # "138": Device_ZWave_Smoke_Alarm,
    # "139": Device_ZWave_Subsystem_Controller,
    # "140": Device_ZWave_Thermostat,
    # "141": Device_ZWave_Toggle,
    # "142": Device_ZWave_Television,
    "143": Device_ZWave_Energy_Meter,
    # "144": Device_ZWave_Pulse_Meter,
    # "145": Device_ZWave_Water_Meter,
    # "146": Device_ZWave_Gas_Meter,
    # "147": Device_ZWave_Binary_Switch,
    # "148": Device_ZWave_Binary_Alarm,
    # "149": Device_ZWave_Aux_Alarm,
    # "150": Device_ZWave_CO2_Alarm,
    # "151": Device_ZWave_CO_Alarm,
    # "152": Device_ZWave_Freeze_Alarm,
    # "153": Device_ZWave_Glass_Break_Alarm,
    # "154": Device_ZWave_Heat_Alarm,
    # "155": Device_ZWave_Motion_Sensor,
    # "156": Device_ZWave_Smoke_Alarm,
    # "157": Device_ZWave_Tamper_Alarm,
    # "158": Device_ZWave_Tilt_Alarm,
    # "159": Device_ZWave_Water_Alarm,
    # "160": Device_ZWave_Door_Window_Alarm,
    # "161": Device_ZWave_Test_Alarm,
    # "162": Device_ZWave_Low_Battery_Alarm,
    # "163": Device_ZWave_CO_End_Of_Life_Alarm,
    # "164": Device_ZWave_Malfunction_Alarm,
    # "165": Device_ZWave_Heartbeat,
    # "166": Device_ZWave_Overheat_Alarm,
    # "167": Device_ZWave_Rapid_Temp_Rise_Alarm,
    # "168": Device_ZWave_Underheat_Alarm,
    # "169": Device_ZWave_Leak_Detected_Alarm,
    # "170": Device_ZWave_Level_Drop_Alarm,
    # "171": Device_ZWave_Replace_Filter_Alarm,
    # "172": Device_ZWave_Intrusion_Alarm,
    # "173": Device_ZWave_Tamper_Code_Alarm,
    # "174": Device_ZWave_Hardware_Failure_Alarm,
    # "175": Device_ZWave_Software_Failure_Alarm,
    # "176": Device_ZWave_Contact_Police_Alarm,
    # "177": Device_ZWave_Contact_Fire_Alarm,
    # "178": Device_ZWave_Contact_Medical_Alarm,
    # "179": Device_ZWave_Wakeup_Alarm,
    # "180": Device_ZWave_Timer,
    # "181": Device_ZWave_Power_Management,
    # "182": Device_ZWave_Appliance,
    # "183": Device_ZWave_Home_Health,
    "184": Device_ZWave_Barrier,
    "185": Device_ZWave_Notification,
    # "186": Device_ZWave_Color_Switch,
    # "187": Device_ZWave_Multilevel_Switch_Off_On,
    # "188": Device_ZWave_Multilevel_Switch_Down_Up,
    # "189": Device_ZWave_Multilevel_Switch_Close_Open,
    # "190": Device_ZWave_Multilevel_Switch_Ccw_Cw,
    # "191": Device_ZWave_Multilevel_Switch_Left_Right,
    # "192": Device_ZWave_Multilevel_Switch_Reverse_Forward,
    # "193": Device_ZWave_Multilevel_Switch_Pull_Push,
    # "194": Device_ZWave_Basic_Set,
    # "195": Device_ZWave_Wall_Controller,
    # "196": Device_ZWave_Barrier_Handle,
    # "197": Device_ZWave_Sound_Switch
}


def get_zwave_device_class(device_info):

    # print ('Z-Wave Device Class')
    # look for specific device device cat that need special handling
    print(device_info)
    # if device_info.category == '1' and device_info.sub_category == '46' and device_info.address_parts [3] == '2': # fanlinc motor
    #     return Device_Insteon_Fan

    # print (device_info.node_def_id.find('KeypadButton') , device_info.address_parts [3])
    # override device cat for keypadlinc dimmer buttons and change to switch type devices
    # if device_info.node_def_id.find('KeypadButton') == 0 and device_info.address_parts [3] != '1': # maybe use node flag
    #     device_info.category = '2'

    # find device class
    # check for specific devcat/subcat

    # print (device_info, device_info.category, device_info.devtype_cat)

    if device_info.devtype_cat in zwave_device_classes:
        device_class = zwave_device_classes[device_info.devtype_cat]
        return device_class
